/**
 * Copyright (C) 2010 <dueni(at)gmx.net>
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package net.java.dev.faces.chain;

import java.io.IOException;
import java.lang.reflect.Constructor;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;
import java.util.Properties;

import javax.el.ELContext;
import javax.faces.context.FacesContext;

import net.java.dev.faces.chain.dynamic.DynamicChain;
import net.java.dev.faces.chain.dynamic.DynamicChainProcessor;
import net.java.dev.faces.chain.handler.CommandHandler;

import org.apache.commons.chain.Command;
import org.apache.commons.chain.config.ConfigParser;
import org.apache.commons.chain.config.ConfigRuleSet;
import org.apache.commons.digester.RuleSet;

/**
 * <code>FacesChainFactory</code> allows to use custom {@link FacesChainFactory} ,
 * {@link FacesChainContext} and/or {@link CommandHandler} by providing the customized
 * implementations and a factory configuration file named {@value #CONFIG_FILE_NAME} on the
 * class-path.
 * <p>
 * Valid configuration options in {@value #CONFIG_FILE_NAME} are:
 * </p>
 * <dl>
 * <dt>{@value #FACES_CHAIN_FACTORY_KEY}</dt>
 * <dd>The class name of a custom FacesChainFactory - must extend {@link FacesChainFactory}.</dd>
 * <dt>{@value #FACES_CHAIN_CONTEXT_KEY}</dt>
 * <dd>The class name of a custom FacesChainContext - the custom FacesChainContext implementation
 * must extend {@link FacesChainContext} and provide a Constructor accepting one parameter of type
 * {@link FacesContext}.</dd>
 * <dt>{@value #COMMAND_HANDLER_KEY}</dt>
 * <dd>The class name of a custom CommandHandler - the custom CommandHandler implementation must
 * extend {@link CommandHandler} and provide a Constructor accepting three parameters of type
 * {@link ELContext}, {@link String} and {@link Command} in this order.</dd>
 * <dt>{@value #CONFIG_RULE_SET_KEY}</dt>
 * <dd>Configure some other class to be used instead of the standard commons-chain ConfigRuleSet -
 * that way you can exchange the classes that will be used for Catalog and/or Chain's.</dd>
 * <dt>{@value #DYNAMIC_CHAIN_PROCESSOR_KEY}</dt>
 * <dd>Configure 0..n DynamicChainProcessor's that can preProcess DynamicChain instances before the
 * execution of the command chain without modifying the initial chain setup. To use that, you also
 * need to set {@link #CONFIG_RULE_SET_KEY}={@link DynamicChain} class name so DynamicChain is used
 * as chain implementation in the catalogs.</dd>
 * </dl>
 * 
 * @author dueni
 * 
 */
public class FacesChainFactory {

	/**
	 * {@link #CONFIG_FILE_NAME} key to configure an alternate {@link FacesChainFactory}.
	 */
	public static final String FACES_CHAIN_FACTORY_KEY = "facesChainFactory";

	/**
	 * {@link #CONFIG_FILE_NAME} key to configure an alternate {@link FacesChainContext}.
	 */
	public static final String FACES_CHAIN_CONTEXT_KEY = "facesChainContext";

	/**
	 * {@link #CONFIG_FILE_NAME} key to configure an alternate {@link CommandHandler}.
	 */
	public static final String COMMAND_HANDLER_KEY = "commandHandler";

	/**
	 * {@link #CONFIG_FILE_NAME} key to configure an alternate {@link ConfigRuleSet}.
	 */
	public static final String CONFIG_RULE_SET_KEY = "configRuleSet";

	/**
	 * {@link #CONFIG_FILE_NAME} key to configure {@link DynamicChainProcessor}. For multiple
	 * processors configuration add ".x" suffix to the property key (e.g.
	 * {@value #DYNAMIC_CHAIN_PROCESSOR_KEY}.1=...,{@value #DYNAMIC_CHAIN_PROCESSOR_KEY}.2=...).
	 */
	public static final String DYNAMIC_CHAIN_PROCESSOR_KEY = "dynamicChainProcessor";

	/**
	 * The properties file name to configure this factory.
	 */
	public static final String CONFIG_FILE_NAME = "faces-chain.properties";

	/**
	 * The factory singleton instance.
	 */
	private static FacesChainFactory factory;

	/**
	 * The constructor to create new instances of FacesChainContext.
	 */
	private Constructor<?> facesChainContextCtor;

	/**
	 * The constructor to create new instances of CommandHandler.
	 */
	private Constructor<?> commandHandlerCtor;

	/**
	 * Optional class name to use a custom RuleSet instead of the default commons-chain
	 * {@link ConfigRuleSet}.
	 */
	private String configRuleSetClassName;

	/**
	 * List of DynamicChainProcessors.
	 */
	private List<DynamicChainProcessor> dynamicChainProcessors;

	/**
	 * Return the singleton FacesChainFactory instance.
	 * 
	 * @return the singleton FacesChainFactory instance.
	 */
	public final static FacesChainFactory getInstance() {
		if (factory == null) {
			Properties config = getFactoryConfig();

			String factoryClass = config.getProperty(FACES_CHAIN_FACTORY_KEY, FacesChainFactory.class
					.getName());
			factory = (FacesChainFactory)FacesChainUtil.newInstance(factoryClass);
			factory.initialize(config);
		}
		return factory;
	}

	/**
	 * Return a new instance of FacesChainContext (or a custom implementation of it) initialized with
	 * the given FacesContext.
	 * 
	 * @param jsf
	 *          the FacesContext to pass to the FacesChainContext constructor.
	 * @return a new instance of FacesChainContext,
	 */
	public FacesChainContext createFacesChainContext(FacesContext jsf) {
		try {
			return (FacesChainContext)facesChainContextCtor.newInstance(jsf);
		} catch (Exception e) {
			throw new IllegalStateException("This should never happpen!", e);
		}
	}

	/**
	 * Return a new instance of CommandHandler (or a custom implementation of it) initialized with the
	 * given ELContext, String and Command.
	 * 
	 * @param elContext
	 *          elContext is the first parameter on the CommandHandler constructor.
	 * @param name
	 *          name is the second parameter on the CommandHandler constructor.
	 * @param command
	 *          command is the third parameter on the CommandHandler constructor.
	 * @return a new instance of CommandHandler.
	 */
	public CommandHandler createCommandHandler(ELContext elContext, String name, Command command) {
		try {
			return (CommandHandler)commandHandlerCtor.newInstance(elContext, name, command);
		} catch (Exception e) {
			throw new IllegalStateException("This should never happpen!", e);
		}
	}

	/**
	 * Return the class name of the ConfigRuleSet to be used by commons-chain's {@link ConfigParser}.
	 * 
	 * @return the class name of the ConfigRuleSet.
	 */
	public String getConfigRuleSetClassName() {
		return configRuleSetClassName;
	}

	/**
	 * Add a DynamicChainProcessor to the list of processors, if that instance is not already on the
	 * list.
	 * 
	 * @param dynamicChainProcessor
	 *          the DynamicChainProcessor to be added.
	 */
	public void addDynamicChainProcessor(DynamicChainProcessor dynamicChainProcessor) {
		if (!getDynamicChainProcessors().contains(dynamicChainProcessor)) {
			getDynamicChainProcessors().add(dynamicChainProcessor);
		}
	}

	/**
	 * Remove the DynamicChainProcessor from the list.
	 * 
	 * @param dynamicChainProcessor
	 *          the DynamicChainProcessor to be removed.
	 */
	public void removeDynamicChainProcessor(DynamicChainProcessor dynamicChainProcessor) {
		getDynamicChainProcessors().remove(dynamicChainProcessor);
	}

	/**
	 * Return the list of {@link DynamicChainProcessor} or an empty list if no processor was added.
	 * 
	 * @return the list of {@link DynamicChainProcessor}.
	 */
	public List<DynamicChainProcessor> getDynamicChainProcessors() {
		if (dynamicChainProcessors == null) {
			dynamicChainProcessors = new ArrayList<DynamicChainProcessor>();
		}
		return dynamicChainProcessors;
	}

	/* -------------------------------------------------------------------- */

	/**
	 * Initialize FacesContextFactory according to the {@value #CONFIG_FILE_NAME} settings.
	 * <p>
	 * The initialization process takes the following optional configuration parameters into account:
	 * </p>
	 * <dl>
	 * <dt><code>facesChainContext</code></dt>
	 * <dd>You can configure another class to be used instead of {@link FacesChainContext} - this
	 * alternative class must extend from {@link FacesChainContext}.</dd>
	 * <dt><code>commandHandler</code></dt>
	 * <dd>You can configure another class to be used instead of {@link CommandHandler} - this
	 * alternative class must extend from {@link CommandHandler}.</dd>
	 * <dt><code>configRuleSet</code></dt>
	 * <dd>Configure your own RuleSet to be used while chain-config.xml files are loaded into
	 * commons-chain's CatalogFactory. For more information about {@link RuleSet} for commons-chain,
	 * see commons-digester and commons-chain.</dd>
	 * </dl>
	 * 
	 * @param config
	 *          the configuration Properties for initialization.
	 */
	protected void initialize(Properties config) {
		// initialize facesChainContextCtor
		String className = config.getProperty(FACES_CHAIN_CONTEXT_KEY, FacesChainContext.class
				.getName());
		facesChainContextCtor = FacesChainUtil.getConstructor(className, FacesContext.class);

		// initialize commandHandlerCtor
		className = config.getProperty(COMMAND_HANDLER_KEY, CommandHandler.class.getName());
		commandHandlerCtor = FacesChainUtil.getConstructor(className, ELContext.class, String.class,
				Command.class);

		// initialize configRuleSetNameClass
		configRuleSetClassName = config.getProperty(CONFIG_RULE_SET_KEY, ConfigRuleSet.class.getName());

		// initialize DynamicChainProcessors
		className = config.getProperty(DYNAMIC_CHAIN_PROCESSOR_KEY);
		if (className != null) {
			addProcessor(className);
		}
		for (Object key : config.keySet()) {
			String param = String.valueOf(key);
			if (param.startsWith(DYNAMIC_CHAIN_PROCESSOR_KEY + ".")) {
				addProcessor(config.getProperty(param));
			}
		}
	}

	/**
	 * Create an instance of the given class name and add it to the list of DynamicChainlistener.
	 * 
	 * @param className
	 *          the class name of a DynamicChainListener.
	 */
	private void addProcessor(String className) {
		DynamicChainProcessor dynamicChainProcessor = (DynamicChainProcessor)FacesChainUtil
				.newInstance(className);
		addDynamicChainProcessor(dynamicChainProcessor);
	}

	/**
	 * Return a Properties object containing the optional configuration settings for FacesChainFactory
	 * - the Properties object is empty if no configuration was found.
	 * 
	 * @param cl
	 *          the class loader used to lookup for the {@value #CONFIG_FILE_NAME} .�
	 * @return a Properties object containing the actual configuration - might be empty if no
	 *         configuration file is present.
	 */
	protected static Properties getFactoryConfig() {
		URL propFile = FacesChainUtil.getClassLoader().getResource(CONFIG_FILE_NAME);
		Properties config = new Properties();
		if (propFile != null) {
			try {
				config.load(propFile.openStream());
			} catch (IOException e) {
				throw new IllegalArgumentException("Invalid content (not valid properties file?) in: "
						+ propFile, e);
			}
		}
		return config;
	}

}
