/**
 * Copyright (C) 2010 <dueni(at)gmx.net>
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package net.java.dev.faces.chain.dynamic;

import java.util.ArrayList;

import net.java.dev.faces.chain.NamedCommand;

import org.apache.commons.chain.Command;

/**
 * <code>CommandList</code> represents a modifiable list of commands - modifications on this list
 * might be done by {@link DynamicChainProcessor}'s. CommandList allows referencing the elements by
 * name for elements implementing the {@link NamedCommand} interface.
 * 
 * @author dueni
 * 
 */
public class CommandList extends ArrayList<Command> {

	private static final long serialVersionUID = 1L;

	public CommandList(Command[] commands) {
		super(commands.length);
		for (Command c : commands) {
			add(c);
		}
	}

	/**
	 * Lookup index position of a Command by name or instance - if the given parameter is a String
	 * name, this name is compared against the name attribute of {@link NamedCommand} instances in the
	 * list, otherwise the {@link ArrayList#indexOf(Object)} is utilized.
	 * 
	 * @param elem
	 *          element to lookup or a String to lookup by {@link NamedCommand#getName()}.
	 */
	@Override
	public int indexOf(Object elem) {
		if (elem instanceof String) {
			String name = (String)elem;
			int m = size();
			for (int i = 0; i < m; i++) {
				Object o = get(i);
				if (o instanceof NamedCommand && name.equals(((NamedCommand)o).getName())) {
					return i;
				}
			}
			return -1;
		}
		return super.indexOf(elem);
	}

	/**
	 * Lookup last index position of a Command by name or instance - if the given parameter is a
	 * String name, this name is compared against the name attribute of {@link NamedCommand} instances
	 * in the list, otherwise the {@link ArrayList#lastIndexOf(Object)} is utilized.
	 * 
	 * @param elem
	 *          element to lookup or a String to lookup by {@link NamedCommand#getName()}.
	 */
	@Override
	public int lastIndexOf(Object elem) {
		if (elem instanceof String) {
			String name = (String)elem;
			for (int i = size() - 1; i > -1; i--) {
				Object item = get(i);
				if (item instanceof NamedCommand && name.equals(((NamedCommand)item).getName())) {
					return i;
				}
			}
			return -1;
		}
		return super.lastIndexOf(elem);
	}

	/**
	 * Returns true if the list contains a Command looked up by name or instance - if the given
	 * parameter is a String name, this name is compared against the name attribute of
	 * {@link NamedCommand} instances in the list, otherwise the {@link ArrayList#contains(Object)} is
	 * utilized.
	 * 
	 * @param elem
	 *          element to lookup or a String to lookup by {@link NamedCommand#getName()}.
	 */
	@Override
	public boolean contains(Object elem) {
		if (elem instanceof String) {
			String name = (String)elem;
			int m = size();
			for (int i = 0; i < m; i++) {
				Object item = get(i);
				if (item instanceof NamedCommand && name.equals(((NamedCommand)item).getName())) {
					return true;
				}
			}
			return false;
		}
		return super.contains(elem);
	}

	/**
	 * Remove a Command by name or instance - if the given parameter is a String name, the first
	 * {@link NamedCommand} with matching name is removed from the list, otherwise
	 * {@link ArrayList#remove(Object)} is utilized.
	 * 
	 * @param elem
	 *          element to lookup or a String to lookup by {@link NamedCommand#getName()}.
	 */
	@Override
	public boolean remove(Object elem) {
		if (elem instanceof String) {
			String name = (String)elem;
			int m = size();
			for (int i = 0; i < m; i++) {
				Object item = get(i);
				if (item instanceof NamedCommand && name.equals(((NamedCommand)item).getName())) {
					remove(i);
					return true;
				}
			}
			return false;
		}
		return super.remove(elem);
	}
}
