/**
 * Copyright (C) 2010 <dueni(at)gmx.net>
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package net.java.dev.faces.chain.handler;

import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import javax.el.ELContext;

import net.java.dev.faces.chain.FacesChainFactory;

import org.apache.commons.chain.Catalog;
import org.apache.commons.chain.CatalogFactory;
import org.apache.commons.chain.Command;

/**
 * <code>CatalogHandler</code> handles access to commons-chain {@link Catalog}'s from expression
 * language (EL) featuring {@link Map} interface based access to
 * {@link CatalogFactory#getCatalog(String)}. If the key passed to the {@link #get(Object)} method
 * cannot be found as a Catalog it will access the default Catalog and look for a {@link Command}
 * with that catalogName.
 * 
 * @author dueni
 * 
 */
public class CatalogHandler extends AbstractChainMap<String, Object> {

	/**
	 * the catalogName of the Catalog.
	 */
	private String catalogName;

	/**
	 * the catalog to handle command lookup on.
	 */
	private Catalog catalog;

	/**
	 * the current ELContext
	 */
	private ELContext elContext;

	/**
	 * Construct a CatalogHandler for the passed Catalog and current ELContext.
	 * 
	 * @param elContext
	 *          current ELContext.
	 * @param catalogName
	 *          the name of this catalog.
	 * @param catalog
	 *          the Catalog to handle Command lookup.
	 */
	public CatalogHandler(ELContext elContext, String catalogName, Catalog catalog) {
		this.elContext = elContext;
		this.catalogName = catalogName;
		this.catalog = catalog;
	}

	/**
	 * Return true if the Catalog contains a command with the given catalogName (key).
	 * 
	 * @return true if a Command for that key is in the Catalog.
	 */
	public boolean containsKey(Object key) {
		Iterator<?> names = catalog.getNames();
		while (names.hasNext()) {
			if (key.equals(names.next())) {
				return true;
			}
		}
		return false;
	}

	/**
	 * Return the Command for the key String or an UnknownCommand to handle misspelled keys.
	 * 
	 * @return the Command for the key or null if key is not a String.
	 */
	public Object get(Object key) {
		CommandHandler commandHandler = null;
		if (key instanceof String) {
			String name = (String)key;
			Command command = catalog.getCommand(name);
			if (command == null) {
				throw new IllegalArgumentException("Command with name '" + name
						+ "' does not exist on Catalog '" + catalogName + "'!");
			}
			commandHandler = FacesChainFactory.getInstance().createCommandHandler(elContext, name,
					command);
		}
		return commandHandler;
	}

	/**
	 * Return true if the Catalog is empty, false otherwise.
	 * 
	 * @return true if the Catalog is empty, false otherwise.
	 */
	public boolean isEmpty() {
		return !catalog.getNames().hasNext();
	}

	/**
	 * Return a set with the command names contained in this Catalog.
	 * 
	 * @return a set with the command names contained in this Catalog.
	 */
	public Set<String> keySet() {
		Set<String> keySet = new HashSet<String>();
		Iterator<?> names = catalog.getNames();
		while (names.hasNext()) {
			keySet.add((String)names.next());
		}
		return keySet;
	}

	/**
	 * Return the number of Commands in the Catalog.
	 * 
	 * @return the number of Commands in the Catalog.
	 */
	public int size() {
		int cnt = 0;
		Iterator<?> names = catalog.getNames();
		while (names.hasNext()) {
			cnt++;
			names.next();
		}
		return cnt;
	}
}
