/**
 * Copyright (C) 2010 <dueni(at)gmx.net>
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package net.java.dev.faces.chain.handler;

import javax.el.ELContext;
import javax.el.ELResolver;
import javax.faces.FacesException;
import javax.faces.context.FacesContext;

import net.java.dev.faces.chain.FacesChainContext;
import net.java.dev.faces.chain.FacesChainFactory;

import org.apache.commons.chain.Command;

/**
 * <code>CommandHandler</code> handles preparation of FacesChainContext and Command execution
 * directly from expression language (EL).
 * 
 * <p>
 * When an EL expression leads to resolve a CommandHandler, the following EL syntax is supported:
 * <code>"#{commandHandler[.beans('beanname1','beanname2')].execute()}"</code> - that means,
 * optionally the beans() method can be used to automatically load JSF managed beans into the
 * FacesChainContext before it is passed to execute. Since beans() again returns the reference to
 * CommandHandler, subsequent calls to beans() would be possible. The <code>execute()</code> method
 * will handle execution of the Command (chain).
 * </p>
 * 
 * @author dueni
 * 
 */
public class CommandHandler {

	/**
	 * The name of the Command.
	 */
	private String name;

	/**
	 * The Command to execute.
	 */
	private Command command;

	/**
	 * The current ELContext.
	 */
	private ELContext elContext;

	/**
	 * The Context to pass to the Command.execute(FacesChainContext).
	 */
	private FacesChainContext facesChainContext;

	/**
	 * Construct a CommandHandler to handle execution on Command. This will also create a new instance
	 * of {@link FacesChainContext} and put it to requestScope on key
	 * {@link FacesChainContext#FACES_CHAIN_CONTEXT_REQ_KEY}.
	 * 
	 * @param elContext
	 *          the current ELContext.
	 * @param name
	 *          the name of the Command to execute.
	 * @param command
	 *          the Command to execute.
	 */
	public CommandHandler(ELContext elContext, String name, Command command) {
		this.elContext = elContext;
		this.name = name;
		this.command = command;
		FacesContext jsf = (FacesContext)elContext.getContext(FacesContext.class);
		facesChainContext = FacesChainFactory.getInstance().createFacesChainContext(jsf);
		facesChainContext.put(ELContext.class, elContext);
		facesChainContext.getRequestScope().put(FacesChainContext.FACES_CHAIN_CONTEXT_REQ_KEY,
				facesChainContext);

	}

	/**
	 * Load multiple JSF managed beans into current FacesChainContext - this method allows to perform
	 * FacesChainContext preparation from EL expression directly - requires EL 2 supporting methods
	 * with variable arguments.
	 * <p>
	 * EL syntax:
	 * <ul compact>
	 * <li><code>"#{commandHandler.beans('beanname1').execute()}"</code></li>
	 * <li><code>"#{commandHandler.beans('beanname1','beanname2').execute()}"</code></li>
	 * <li><code>"#{commandHandler.beans('beanname1','bean3',bean4').execute()}"</code></li>
	 * </ul>
	 * </p>
	 * 
	 * @param beanNames
	 *          0...n names of managed beans to load into FacesChainContext.
	 * @return this.
	 */
	public CommandHandler beans(String... beanNames) {
		for (String beanName : beanNames) {
			bean(beanName);
		}
		return this;
	}

	/**
	 * Load one JSF managed bean into current FacesChainContext - this method allows to perform
	 * {@link FacesChainContext} preparation from EL expression directly (this method also supports EL
	 * before version 2). Load multiple beans into FacesChainContext can be done by multiple calls to
	 * this method.
	 * <p>
	 * EL syntax:
	 * <ul compact>
	 * <li><code>"#{commandHandler.bean('beanname1').execute()}"</code></li>
	 * <li><code>"#{commandHandler.bean('beanname1').bean('beanname2').execute()}"</code></li>
	 * </ul>
	 * 
	 * @param name
	 *          name of managed bean to load into FacesChainContext.
	 * @return this.
	 */
	public CommandHandler bean(String name) {
		ELResolver resolver = elContext.getELResolver();

		Object result = resolver.getValue(elContext, null, name);
		facesChainContext.put(name, result);
		return this;
	}

	/**
	 * Put some EL variable to FacesChainContext before the command is executed.
	 * <p>
	 * EL syntax:
	 * <ul compact>
	 * <li><code>"#{commandHandler.put('name',value).execute()}"</code></li>
	 * <li><code>"#{commandHandler.bean('beanname1').put('name',value).execute()}"</code></li>
	 * </ul>
	 * </p>
	 * 
	 * @param name
	 *          the key to put value in FacesChainContext.
	 * @param value
	 *          the object to put in FacesChainContext.
	 * @return this.
	 */
	public CommandHandler put(String name, Object value) {
		facesChainContext.put(name, value);
		return this;
	}

	/**
	 * Allow to preset the outcome of the command execution - this outcome will be used for navigation
	 * if none of the commands changes the outcome and no exception is thrown.
	 * <p>
	 * EL syntax:
	 * <ul compact>
	 * <li><code>"#{commandHandler.outcome('done').execute()}"</code></li>
	 * </ul>
	 * </p>
	 * 
	 * @param outcome
	 *          the outcome to preset.
	 * @return this.
	 */
	public CommandHandler outcome(Object outcome) {
		facesChainContext.setOutcome(outcome);
		return this;
	}

	/**
	 * Execute the command (chain) by passing the prepared FacesChainContext. After execution of the
	 * command/chain, the outcome set on FacesChainContext will be passed back to the action.
	 * 
	 * @return the outcome that might have been set on FacesChainContext by {@link #outcome(Object)}
	 *         or any of the executed commands.
	 */
	public String execute() {
		try {
			command.execute(facesChainContext);
			return (String)facesChainContext.getOutcome();
		} catch (Exception e) {
			throw new FacesException("Error while executing command/chain named '" + name + "':", e);
		}
	}

}
